/*
**    3ETI Projet CPE Lyon
**    Copyright (C) 2011 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <v4.hpp>

#include <string_helper.hpp>
#include <cmath>
#include <v3.hpp>


namespace cpe
{
    v4::v4():internal_x(0),internal_y(0),internal_z(0),internal_w(0){}
    v4::v4(const double& x,const double& y,const double& z,const double& w):internal_x(x),internal_y(y),internal_z(z),internal_w(w){}
    v4::v4(const v3& xyz,const double& _w):internal_x(xyz.x()),internal_y(xyz.y()),internal_z(xyz.z()),internal_w(_w){}
    const double& v4::x() const{return internal_x;}
    double& v4::x(){return internal_x;}
    const double& v4::y() const{return internal_y;}
    double& v4::y(){return internal_y;}
    const double& v4::z() const{return internal_z;}
    double& v4::z(){return internal_z;}
    const double& v4::w() const{return internal_w;}
    double& v4::w(){return internal_w;}

    const double& v4::operator[](const size_t& k) const{assert_size(k);return (k==0?internal_x:(k==1?internal_y:(k==2?internal_z:internal_w)));}
    double& v4::operator[](const size_t& k){assert_size(k);return (k==0?internal_x:(k==1?internal_y:(k==2?internal_z:internal_w)));}
    const double& v4::operator()(const size_t& k) const{assert_size(k);return (k==0?internal_x:(k==1?internal_y:(k==2?internal_z:internal_w)));}
    double& v4::operator()(const size_t& k){assert_size(k);return (k==0?internal_x:(k==1?internal_y:(k==2?internal_z:internal_w)));}

    void v4::set_zero(){internal_x=0;internal_y=0;internal_z=0;internal_w=0;}
    double v4::dot(const v4& p) const{return internal_x*p.x()+internal_y*p.y()+internal_z*p.z()+internal_w*p.w();}
    double v4::norm() const{return std::sqrt(norm2());}
    double v4::norm2() const{return internal_x*internal_x+internal_y*internal_y+internal_z*internal_z+internal_w*internal_w;}

    v4 operator+(const v4& p1,const v4& p2){return v4(p1.x()+p2.x(),p1.y()+p2.y(),p1.z()+p2.z(),p1.w()+p2.w());}
    v4 operator+(const double& s,const v4& p){return v4(s+p.x(),s+p.y(),s+p.z(),s+p.w());}
    v4 operator+(const v4& p,const double& s){return s+p;}
    v4 operator-(const v4& p1,const v4& p2){return v4(p1.x()-p2.x(),p1.y()-p2.y(),p1.z()-p2.z(),p1.w()-p2.w());}
    v4 operator-(const double& s,const v4& p){return v4(s-p.x(),s-p.y(),s-p.z(),s-p.w());}
    v4 operator-(const v4& p,const double& s){return v4(p.x()-s,p.y()-s,p.z()-s,p.w()-s);}
    v4 operator*(const double& s,const v4& p){return v4(s*p.x(),s*p.y(),s*p.z(),s*p.w());}
    v4 operator*(const v4& p,const double& s){return s*p;}
    v4 operator/(const v4& p,const double& s){return v4(p.x()/s,p.y()/s,p.z()/s,p.w()/s);}
    v4& v4::operator+=(const v4& p) {internal_x+=p.x();internal_y+=p.y();internal_z+=p.z();internal_w+=p.w(); return *this;}
    v4& v4::operator+=(const double& s) {internal_x+=s;internal_y+=s;internal_z+=s;internal_w+=s; return *this;}
    v4& v4::operator-=(const v4& p) {internal_x-=p.x();internal_y-=p.y();internal_z-=p.z();internal_w-=p.w(); return *this;}
    v4& v4::operator-=(const double& s) {internal_x-=s;internal_y-=s;internal_z-=s;internal_w-=s; return *this;}
    v4& v4::operator*=(const double& s) {internal_x*=s;internal_y*=s;internal_z*=s;internal_w*=s; return *this;}
    v4& v4::operator/=(const double& s) {internal_x/=s;internal_y/=s;internal_z/=s;internal_w/=s; return *this;}
    v4 v4::operator-() const {return v4(-internal_x,-internal_y,-internal_z,-internal_w);}

    v4 v4::product_compontentwise(const v4& p) const {return v4(internal_x*p.x(),internal_y*p.y(),internal_z*p.z(),internal_w*p.w());}
    v4& v4::product_compontentwise_internal(const v4& p) {internal_x*=p.x();internal_y*=p.y();internal_z*=p.z();internal_w*=p.w(); return *this;}
    std::string v4::to_string() const {return cpe::string_converter::to_string(internal_x)+" "+cpe::string_converter::to_string(internal_y)+" "+cpe::string_converter::to_string(internal_z);}
    std::ostream& operator<<(std::ostream& stream,const v4& p) {stream<<"("<<p.to_string()<<")";return stream;}

    void v4::assert_size(const size_t& k) const throw(std::exception) {if(k>3) throw std::string("v4::assert_size("+cpe::string_converter::to_string(k)+") k>3");}

}
