
/** Projet ETI3 - CPE Lyon - 2010-2011 */



#include <glut_helper.hpp>


#include <GL/glut.h>
#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <string>
#include <errno.h>
#include <vector>





//draw the model
void draw_scene();
//load the model
void load_model();
//compute a random number
double get_rand();



//glut functions
int setup_callback();
static void display_callback();
static void reshape_callback (int width, int height);
static void keyboard_callback(unsigned char key,int x,int y);
static void idle_callback(int value);
static void motion_callback (int x, int y);
static void mouse_callback (int button, int action, int x, int y);



using namespace cpe;




//GLOBAL VARIABLES
//************************************//
unsigned int time_anim=0;
unsigned int counter=0;
navigator_tool nav;





//load the model
void load_model()
{
    //preprocessing
    std::cout<<"<DO PREPROCESSING ...>"<<std::endl;



    //... do all my pre-processing stuff


    std::cout<<"</DO PREPROCESSING> "<<std::endl;
}


//Draw the scene
void draw_scene()
{
    ++time_anim;

    glEnable(GL_LIGHTING);

    //... do all my drawing




    //some examples :

    //*************************************//
    //draw triangle
    //*************************************//

    glBegin(GL_TRIANGLES);
    glNormal3d(0,0,1);
    glColor3d(1,0,0);
    glVertex3d(0,0.4,0);

    glColor3d(0,0,1);
    glVertex3d(0.4,0.4,0.3);

    glColor3d(0,1,0);
    glVertex3d(0,0.6,0);
    glEnd();



    //*************************************//
    //draw 1D function
    //*************************************//

    //compute values
    std::vector<double> z;
    std::vector<double> dz;
    std::vector<double> n;
    unsigned int N=4*1*200+1;
    for(unsigned int k=0;k<N;++k)
    {
        double alpha=k/double(N-1);
        z.push_back(0.3*exp(-5*alpha)*sin(16*alpha*M_PI+time_anim/50.0));

        //normal
        dz.push_back(0.3*exp(-5*alpha)*(-5*sin(16*alpha*M_PI)+16*M_PI*cos(16*alpha*M_PI+time_anim/50.0)));
        n.push_back(std::sqrt(1+dz[k]*dz[k]));
    }

    //draw surface with quads
    glEnable(GL_POLYGON_OFFSET_FILL);
    glPolygonOffset(2,2);
    glBegin(GL_QUADS);
    for(unsigned int k=0;k<N-1;k++)
    {
        double alpha=k/double(N-1);
        double alpha2=alpha+1.0/double(N-1);

        glColor3d(0.9,0.9,0.9);

        glColor3d(1-alpha,alpha,0);
        glNormal3d(dz[k]/n[k],0,-1/n[k]);
        glVertex3d(alpha,0,z[k]);

        glColor3d(1-alpha2,alpha2,0);
        glNormal3d(dz[k+1]/n[k+1],0,-1/n[k+1]);
        glVertex3d(alpha2,0,z[k+1]);

        glNormal3d(dz[k+1]/n[k+1],0,-1/n[k+1]);
        glVertex3d(alpha2,0.3,z[k+1]);

        glNormal3d(dz[k]/n[k],0,-1/n[k]);
        glVertex3d(alpha,0.3,z[k]);

    }
    glEnd();
    glDisable(GL_POLYGON_OFFSET_FILL);

    //draw boundaries
    glColor3d(0,0,0);
    glLineWidth(2.0);
    glBegin(GL_LINES);
    for(unsigned int k=0;k<N-1;++k)
    {
        double alpha=k/double(N-1);
        double alpha2=alpha+1.0/double(N-1);

        glVertex3d(alpha,0,z[k]);
        glVertex3d(alpha2,0,z[k+1]);

        glVertex3d(alpha,0.3,z[k]);
        glVertex3d(alpha2,0.3,z[k+1]);
    }
    for(unsigned int k=0;k<N;++k)
    {
        if(k==0 || k==N-1)
            glColor3d(0,0,0);
        else
            glColor3d(0.3,0.3,0.3);

        double alpha=k/double(N-1);
        if( k%((N-1)/4/8)==0)
        {
            glVertex3d(alpha,0.0,z[k]);
            glVertex3d(alpha,0.3,z[k]);
        }
    }
    glEnd();





}















int main(int argc, char *argv[])
{
    std::cout<<"**hello**\n"<<std::endl;
    std::cout<<"run "<<argv[0]<<" with "<<argc-1<<" parameters ... \n"<<std::endl;


  // ************* //
  // start OpenGL
  // ************* //

  //window size
  nav.screen_size_x()=800;
  nav.screen_size_y()=800;



  //init opengl & glut
  setup_opengl(argc,argv,nav);
  //setup callback functions
  setup_callback();





  // ************* //
  // RUN
  // ************* //
  print_help_start();

  //do start once
  load_model();

  //glut loop
  std::cout<<"<RUN MAIN LOOP ...>"<<std::endl;
  glutMainLoop ();


  return 0;
}



// *******************//
// Helper
// *******************//

//just a rand number \in [0,1]
double get_rand()
{
  int precision=50000;
  return double(rand()%precision)/double(precision);
}






//*******************//
//GLUT STUFF
//*******************//


//glut functions
int setup_callback()
{

    //callback
    glutDisplayFunc (display_callback);
    glutReshapeFunc (reshape_callback);
    glutKeyboardFunc(keyboard_callback);
    glutTimerFunc(33,idle_callback,1);
    glutMotionFunc (motion_callback);
    glutMouseFunc (mouse_callback);

    return 0;
}



//display
static void display_callback()
{
    display_manager_begin(nav);
    draw_scene();
    display_manager_end(nav,counter);
}


static void reshape_callback (int width, int height) {reshape_manager(width,height,nav);}
static void idle_callback(int value) {value=1;glutPostRedisplay ();glutTimerFunc( 20, idle_callback, 1);}
static void keyboard_callback(unsigned char key,int x,int y) {keyboard_manager(key,x,y,nav);}
static void motion_callback (int x, int y) {motion_manager(x,y,nav);}
static void mouse_callback (int button, int action, int x, int y) {mouse_manager(button,action,x,y,nav);}












